@@ -3,11 +3,14 @@ import functools
 import inspect
 import sys
 import time
+import uuid
 import calendar
 import unittest
 import platform
+import warnings
 
 from dateutil import parser
+from dateutil.tz import tzlocal
 
 real_time = time.time
 real_localtime = time.localtime
@@ -16,6 +19,16 @@ real_strftime = time.strftime
 real_date = datetime.date
 real_datetime = datetime.datetime
 
+try:
+    real_uuid_generate_time = uuid._uuid_generate_time
+except ImportError:
+    real_uuid_generate_time = None
+
+try:
+    real_uuid_create = uuid._UuidCreate
+except ImportError:
+    real_uuid_create = None
+
 try:
     import copy_reg as copyreg
 except ImportError:
@@ -29,7 +42,7 @@ def with_metaclass(meta, *bases):
 
 _is_cpython = (
     hasattr(platform, 'python_implementation') and
-    platform.python_implementation().lower == "cpython"
+    platform.python_implementation().lower() == "cpython"
 )
 
 
@@ -168,15 +181,18 @@ class FakeDatetime(with_metaclass(FakeDatetimeMeta, real_datetime, FakeDate)):
         else:
             return result
 
-    def astimezone(self, tz):
+    def astimezone(self, tz=None):
+        if tz is None:
+            tz = tzlocal()
         return datetime_to_fakedatetime(real_datetime.astimezone(self, tz))
 
     @classmethod
     def now(cls, tz=None):
+        now = cls._time_to_freeze() or real_datetime.now()
         if tz:
-            result = tz.fromutc(cls._time_to_freeze().replace(tzinfo=tz)) + datetime.timedelta(hours=cls._tz_offset())
+            result = tz.fromutc(now.replace(tzinfo=tz)) + datetime.timedelta(hours=cls._tz_offset())
         else:
-            result = cls._time_to_freeze() + datetime.timedelta(hours=cls._tz_offset())
+            result = now + datetime.timedelta(hours=cls._tz_offset())
         return datetime_to_fakedatetime(result)
 
     def date(self):
@@ -188,12 +204,13 @@ class FakeDatetime(with_metaclass(FakeDatetimeMeta, real_datetime, FakeDate)):
 
     @classmethod
     def utcnow(cls):
-        result = cls._time_to_freeze()
+        result = cls._time_to_freeze() or real_datetime.utcnow()
         return datetime_to_fakedatetime(result)
 
     @classmethod
     def _time_to_freeze(cls):
-        return cls.times_to_freeze[-1]()
+        if cls.times_to_freeze:
+            return cls.times_to_freeze[-1]()
 
     @classmethod
     def _tz_offset(cls):
@@ -236,6 +253,23 @@ def pickle_fake_datetime(datetime_):
     )
 
 
+def _parse_time_to_freeze(time_to_freeze_str):
+    """Parses all the possible inputs for freeze_time
+    :returns: a naive ``datetime.datetime`` object
+    """
+    if time_to_freeze_str is None:
+        time_to_freeze_str = datetime.datetime.utcnow()
+
+    if isinstance(time_to_freeze_str, datetime.datetime):
+        time_to_freeze = time_to_freeze_str
+    elif isinstance(time_to_freeze_str, datetime.date):
+        time_to_freeze = datetime.datetime.combine(time_to_freeze_str, datetime.time())
+    else:
+        time_to_freeze = parser.parse(time_to_freeze_str)
+
+    return convert_to_timezone_naive(time_to_freeze)
+
+
 class TickingDateTimeFactory(object):
 
     def __init__(self, time_to_freeze, start):
@@ -257,19 +291,18 @@ class FrozenDateTimeFactory(object):
     def tick(self, delta=datetime.timedelta(seconds=1)):
         self.time_to_freeze += delta
 
+    def move_to(self, target_datetime):
+        """Moves frozen date to the given ``target_datetime``"""
+        target_datetime = _parse_time_to_freeze(target_datetime)
+        delta = target_datetime - self.time_to_freeze
+        self.tick(delta=delta)
+
 
 class _freeze_time(object):
 
     def __init__(self, time_to_freeze_str, tz_offset, ignore, tick):
-        if isinstance(time_to_freeze_str, datetime.datetime):
-            time_to_freeze = time_to_freeze_str
-        elif isinstance(time_to_freeze_str, datetime.date):
-            time_to_freeze = datetime.datetime.combine(time_to_freeze_str, datetime.time())
-        else:
-            time_to_freeze = parser.parse(time_to_freeze_str)
-        time_to_freeze = convert_to_timezone_naive(time_to_freeze)
 
-        self.time_to_freeze = time_to_freeze
+        self.time_to_freeze = _parse_time_to_freeze(time_to_freeze_str)
         self.tz_offset = tz_offset
         self.ignore = tuple(ignore)
         self.tick = tick
@@ -310,7 +343,9 @@ class _freeze_time(object):
         else:
 
             seen = set()
-            for base_klass in klass.mro():
+
+            klasses = klass.mro() if hasattr(klass, 'mro') else [klass] + list(klass.__bases__)
+            for base_klass in klasses:
                 for (attr, attr_value) in base_klass.__dict__.items():
                     if attr.startswith('_') or attr in seen:
                         continue
@@ -349,6 +384,8 @@ class _freeze_time(object):
         time.localtime = fake_localtime
         time.gmtime = fake_gmtime
         time.strftime = fake_strftime
+        uuid._uuid_generate_time = None
+        uuid._UuidCreate = None
 
         copyreg.dispatch_table[real_datetime] = pickle_fake_datetime
         copyreg.dispatch_table[real_date] = pickle_fake_date
@@ -371,25 +408,28 @@ class _freeze_time(object):
         # Save the current loaded modules
         self.modules_at_start = set(sys.modules.keys())
 
-        for mod_name, module in list(sys.modules.items()):
-            if mod_name is None or module is None:
-                continue
-            elif mod_name.startswith(self.ignore):
-                continue
-            elif (not hasattr(module, "__name__") or module.__name__ in ('datetime', 'time')):
-                continue
-            for module_attribute in dir(module):
-                if module_attribute in real_names:
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore')
+
+            for mod_name, module in list(sys.modules.items()):
+                if mod_name is None or module is None:
                     continue
-                try:
-                    attribute_value = getattr(module, module_attribute)
-                except (ImportError, AttributeError, TypeError):
-                    # For certain libraries, this can result in ImportError(_winreg) or AttributeError (celery)
+                elif mod_name.startswith(self.ignore):
                     continue
-                fake = fakes.get(id(attribute_value))
-                if fake:
-                    setattr(module, module_attribute, fake)
-                    add_change((module, module_attribute, attribute_value))
+                elif (not hasattr(module, "__name__") or module.__name__ in ('datetime', 'time')):
+                    continue
+                for module_attribute in dir(module):
+                    if module_attribute in real_names:
+                        continue
+                    try:
+                        attribute_value = getattr(module, module_attribute)
+                    except (ImportError, AttributeError, TypeError):
+                        # For certain libraries, this can result in ImportError(_winreg) or AttributeError (celery)
+                        continue
+                    fake = fakes.get(id(attribute_value))
+                    if fake:
+                        setattr(module, module_attribute, fake)
+                        add_change((module, module_attribute, attribute_value))
 
         datetime.datetime.times_to_freeze.append(time_to_freeze)
         datetime.datetime.tz_offsets.append(self.tz_offset)
@@ -417,32 +457,38 @@ class _freeze_time(object):
             # Restore modules loaded after start()
             modules_to_restore = set(sys.modules.keys()) - self.modules_at_start
             self.modules_at_start = set()
-            for mod_name in modules_to_restore:
-                module = sys.modules.get(mod_name, None)
-                if mod_name is None or module is None:
-                    continue
-                elif mod_name.startswith(self.ignore):
-                    continue
-                elif (not hasattr(module, "__name__") or module.__name__ in ('datetime', 'time')):
-                    continue
-                for module_attribute in dir(module):
-                    if module_attribute in self.fake_names:
+            with warnings.catch_warnings():
+                warnings.simplefilter('ignore')
+                for mod_name in modules_to_restore:
+                    module = sys.modules.get(mod_name, None)
+                    if mod_name is None or module is None:
                         continue
-                    try:
-                        attribute_value = getattr(module, module_attribute)
-                    except (ImportError, AttributeError, TypeError):
-                        # For certain libraries, this can result in ImportError(_winreg) or AttributeError (celery)
+                    elif mod_name.startswith(self.ignore):
                         continue
+                    elif (not hasattr(module, "__name__") or module.__name__ in ('datetime', 'time')):
+                        continue
+                    for module_attribute in dir(module):
 
-                    real = self.reals.get(id(attribute_value))
-                    if real:
-                        setattr(module, module_attribute, real)
+                        if module_attribute in self.fake_names:
+                            continue
+                        try:
+                            attribute_value = getattr(module, module_attribute)
+                        except (ImportError, AttributeError, TypeError):
+                            # For certain libraries, this can result in ImportError(_winreg) or AttributeError (celery)
+                            continue
+
+                        real = self.reals.get(id(attribute_value))
+                        if real:
+                            setattr(module, module_attribute, real)
 
         time.time = time.time.previous_time_function
         time.gmtime = time.gmtime.previous_gmtime_function
         time.localtime = time.localtime.previous_localtime_function
         time.strftime = time.strftime.previous_strftime_function
 
+        uuid._uuid_generate_time = real_uuid_generate_time
+        uuid._UuidCreate = real_uuid_create
+
     def decorate_callable(self, func):
         def wrapper(*args, **kwargs):
             with self:
@@ -457,15 +503,15 @@ class _freeze_time(object):
         return wrapper
 
 
-def freeze_time(time_to_freeze, tz_offset=0, ignore=None, tick=False):
+def freeze_time(time_to_freeze=None, tz_offset=0, ignore=None, tick=False):
     # Python3 doesn't have basestring, but it does have str.
     try:
         string_type = basestring
     except NameError:
         string_type = str
 
-    if not isinstance(time_to_freeze, (string_type, datetime.date)):
-        raise TypeError(('freeze_time() expected a string, date instance, or '
+    if not isinstance(time_to_freeze, (type(None), string_type, datetime.date)):
+        raise TypeError(('freeze_time() expected None, a string, date instance, or '
                          'datetime instance, but got type {0}.').format(type(time_to_freeze)))
     if tick and not _is_cpython:
         raise SystemError('Calling freeze_time with tick=True is only compatible with CPython')
@@ -474,6 +520,8 @@ def freeze_time(time_to_freeze, tz_offset=0, ignore=None, tick=False):
         ignore = []
     ignore.append('six.moves')
     ignore.append('django.utils.six.moves')
+    ignore.append('threading')
+    ignore.append('Queue')
     return _freeze_time(time_to_freeze, tz_offset, ignore, tick)
 
 
@@ -502,4 +550,6 @@ except ImportError:
     pass
 else:
     pymysql.converters.encoders[FakeDate] = pymysql.converters.encoders[real_date]
+    pymysql.converters.conversions[FakeDate] = pymysql.converters.encoders[real_date]
     pymysql.converters.encoders[FakeDatetime] = pymysql.converters.encoders[real_datetime]
+    pymysql.converters.conversions[FakeDatetime] = pymysql.converters.encoders[real_datetime]
